<?php

namespace Drupal\config_export_import\Form;

use Drupal\Core\Url;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Component\Serialization\Exception\InvalidDataTypeException;
use Drupal\Component\Serialization\Yaml;
/**
 * Class ContentExportSettingForm.
 */
class ConfigExportForm extends FormBase {

//  /**
//   * {@inheritdoc}
//   */
  protected function getEditableConfigNames() {
    return [
      'config_export_import.config_export',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'config_export_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

      $helper = \Drupal::service('config_export_import.manager');
      $query = $this->getRequest()->query->all();
      $form_state->setMethod('GET');
      $form['key'] = [
          '#type' => 'textfield',
          '#required' => TRUE,
          '#title' => $this->t('Config search by key'),
          '#attributes' => ['name' => 'key'],
          '#default_value' => isset($query['key'])?$query['key']:''
      ];
      $form['path'] = [
          '#type' => 'textfield',
          '#title' => $this->t('Config path to export'),
          '#attributes' => ['name' => 'path'],
          '#description' => 'For example : /sites/default/files/config',
          '#default_value' => isset($query['path'])?$query['path']:''
      ];
      $header = [
          'number' =>  t('Number'),
          'config_name' => t('Config name')
      ];
      $output = [];
      $filter = null ;
      if( isset($query['op']) && $query['op']=='Search' && isset($query['path']) && $query['path'] !='' ) {
          $form['help'] = [
              '#type' => 'item',
              '#title' => t('Seleeted key and path'),
              '#markup' => 'Key : '.$query['key'].'<br/> Path : '.$query['path']
          ];
      }
      if(isset($query['key']) && $query['key'] !=''){
          $filter = $query['key'];
      }
      $form['actions'] = ['#type' => 'actions'];
      $form['actions']['submit'] = [
          '#type' => 'submit',
          '#value' => $this->t('Search'),
      ];

      if(isset($query['op']) && $query['op']=='Search' && isset($query['path']) && $query['path'] !=''){
          $form['actions']['export'] = [
              '#type' => 'submit',
              '#value' => 'Export'
          ];
      }
      $output = [];
      if($filter) {
          $results = $helper->getConfigContains($filter);
          foreach ($results as $key => $result) {
              $output[] = [
                  'id' => $key+1 ,
                  'config_name' =>  $result
              ];

          }
      }
      $form['table'] = array(
          '#type' => 'table',
          '#weight'=> 999,
          '#header' => $header,
          '#rows' => $output,
          '#empty' => $this->t('No variables found')
      );
      if(isset($query['op'])
          && $query['op']=='Export' && isset($query['path']) && $query['path'] !=''){
          $path = $query['path'];
          foreach ($results as $key => $result){
              $helper->exportConfig($result,$path);
          }
      }
      return $form ;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

  }


  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

  }

}
