<?php

namespace Drupal\config_export_import\Form;

use Drupal\Core\Url;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Component\Serialization\Exception\InvalidDataTypeException;
use Drupal\Component\Serialization\Yaml;
use Drupal\Core\Config\FileStorage;
/**
 * Class ConfigImportForm.
 */
class ConfigImportForm extends FormBase {



//  /**
//   * {@inheritdoc}
//   */
  protected function getEditableConfigNames() {
    return [
      'config_export_import.config_import',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'config_import_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
      $helper = \Drupal::service('config_export_import.manager');
      $query = $this->getRequest()->query->all();
      $form_state->setMethod('GET');
      $form['key'] = [
          '#type' => 'textfield',
          '#title' => $this->t('Config search by key'),
          '#attributes' => ['name' => 'key'],
          '#default_value' => isset($query['key'])?$query['key']:'',
          '#description' => 'Make empty to get all'
      ];
      $form['path'] = [
          '#type' => 'textfield',
          '#title' => $this->t('Config path location'),
          '#attributes' => ['name' => 'path'],
          '#description' => 'For example : /sites/default/files/config',
          '#default_value' => isset($query['path'])?$query['path']:''
      ];
      $header = [
          'number' =>  t('Number'),
          'config_name' => t('Config name')
      ];
      $output = [];
      if(isset($query['op']) && $query['op']=='Search'
          && isset($query['path']) && $query['path'] !=''){
          $key = '';
          if(isset($query['key'])){
              $key = $query['key'] ;
          }
          $form['help'] = [
              '#type' => 'item',
              '#title' => t('Selected key and path'),
              '#markup' => 'Key:'.$key.'<br/> Path : '.$query['path'],
          ];

      }
      $form['actions'] = ['#type' => 'actions'];
      $form['actions']['submit'] = [
          '#type' => 'submit',
          '#value' => 'Search',

      ];

      if(isset($query['path']) && $query['path']!=''){
          $filter = null ;
          $path = $query['path'];
          $config_path = DRUPAL_ROOT .$path;
          $results = $helper->readDirectory($config_path,'yml');
          $form['actions']['import'] = [
              '#type' => 'submit',
              '#value' => 'Import'
          ];
          if(isset($query['key']) && $query['key'] !=''){
              foreach ($results as $key => $result) {
                  $config_name = basename($result,'.yml') ;
                  $filter = $query['key'] ;
                  if (is_string($filter) && strpos($config_name, $filter) !== false) {
                      $output[] = [
                          'id' => $key+1 ,
                          'config_name' =>  $config_name
                      ];
                  }
              }
          }else{
              foreach ($results as $key => $result) {
                  $config_name = basename($result,'.yml') ;
                      $output[] = [
                          'id' => $key+1 ,
                          'config_name' =>  $config_name
                      ];
              }
          }

          $form['table'] = array(
              '#type' => 'table',
              '#weight'=> 999,
              '#header' => $header,
              '#rows' => $output,
              '#empty' => $this->t('No variables found')
          );
      }
      if(isset($query['op'])
          && $query['op']=='Import' && isset($query['path'])
          && $query['path']!=''){
          $path = $query['path'];
          $config_path = DRUPAL_ROOT .$path;
          $results = $helper->readDirectory($config_path,'yml');
          if(isset($query['key']) && $query['key'] !=''){
              foreach ($results as $key => $result){
                  $config_name = basename($result,'.yml') ;
                  $filter = $query['key'] ;
                  if ((is_string($filter) && strpos($config_name, $filter) !== false)) {
                       $helper->importConfig($result);
                  }

              }
          }else{
              foreach ($results as $key => $result){
                  $helper->importConfig($result);
              }
          }
      }
    return $form ;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

  }


}
